@@ -8,9 +8,14 @@ class UriSplitTest(unittest.TestCase):
     def check(self, uri, parts):
         result = urisplit(uri)
         self.assertEqual(result, parts)
+        self.assertEqual(result.geturi(), uri)
+        self.assertEqual(result.getscheme(), parts[0])
+        self.assertEqual(result.getauthority(), parts[1])
+        self.assertEqual(result.getpath(), parts[2])
+        self.assertEqual(result.getquery(), parts[3])
+        self.assertEqual(result.getfragment(), parts[4])
         for r, p in zip(result, parts):
             self.assertIsInstance(r, type(p))
-        self.assertEqual(result.geturi(), uri)
         self.assertIsInstance(result.geturi(), type(uri))
 
     def test_rfc3986(self):
@@ -60,6 +65,7 @@ class UriSplitTest(unittest.TestCase):
 
         uri = 'foo://user@example.com:8042/over/there?name=ferret#nose'
         result = urisplit(uri)
+
         self.assertEqual(result.scheme, 'foo')
         self.assertEqual(result.authority, 'user@example.com:8042')
         self.assertEqual(result.path, '/over/there')
@@ -67,11 +73,21 @@ class UriSplitTest(unittest.TestCase):
         self.assertEqual(result.fragment, 'nose')
         self.assertEqual(result.userinfo, 'user')
         self.assertEqual(result.host, 'example.com')
-        self.assertEqual(result.port, 8042)
+        self.assertEqual(result.port, '8042')
+
+        self.assertEqual(result.getscheme(), 'foo')
+        self.assertEqual(result.getauthority(), 'user@example.com:8042')
+        self.assertEqual(result.getpath(), '/over/there')
+        self.assertEqual(result.getquery(), 'name=ferret')
+        self.assertEqual(result.getfragment(), 'nose')
+        self.assertEqual(result.getuserinfo(), 'user')
+        self.assertEqual(result.gethost(), 'example.com')
+        self.assertEqual(result.getport(), 8042)
         self.assertEqual(result.geturi(), uri)
 
         uri = 'urn:example:animal:ferret:nose'
         result = urisplit(uri)
+
         self.assertEqual(result.scheme, 'urn')
         self.assertEqual(result.authority, None)
         self.assertEqual(result.path, 'example:animal:ferret:nose')
@@ -81,3 +97,18 @@ class UriSplitTest(unittest.TestCase):
         self.assertEqual(result.host, None)
         self.assertEqual(result.port, None)
         self.assertEqual(result.geturi(), uri)
+
+        self.assertEqual(result.getscheme(), 'urn')
+        self.assertEqual(result.getauthority(), None)
+        self.assertEqual(result.getpath(), 'example:animal:ferret:nose')
+        self.assertEqual(result.getquery(), None)
+        self.assertEqual(result.getfragment(), None)
+        self.assertEqual(result.getuserinfo(), None)
+        self.assertEqual(result.gethost(), None)
+        self.assertEqual(result.getport(), None)
+        self.assertEqual(result.geturi(), uri)
+
+        uri = 'foo_bar://example.com/'
+        result = urisplit(uri)
+        with self.assertRaises(ValueError):
+            result.getscheme()
