@@ -6,11 +6,11 @@ commonly used functions of the Python Standard Library :mod:`urlparse`
 module.
 
 """
-from collections import namedtuple
+import collections
 import re
 import urllib
 
-__version__ = '0.2.0'
+__version__ = '0.3.0'
 
 RE = re.compile(r"""
 (?:(?P<scheme>[^:/?#]+):)?      # scheme
@@ -20,18 +20,18 @@ RE = re.compile(r"""
 (?:\#(?P<fragment>.*))?         # fragment
 """, flags=(re.VERBOSE))
 """Regular expression for splitting a well-formed URI into its
-components.
+components as specified in RFC 3986 Appendix B.
 
 """
 
 GEN_DELIMS = ':/?#[]@'
-"""General delimiting characters."""
+"""General delimiting characters specified in RFC 3986."""
 
 SUB_DELIMS = "!$&'()*+,;="
-"""Subcomponent delimiting characters."""
+"""Subcomponent delimiting characters specified in RFC 3986."""
 
 RESERVED = GEN_DELIMS + SUB_DELIMS
-"""Reserved characters."""
+"""Reserved characters specified in RFC 3986."""
 
 UNRESERVED = (
     'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
@@ -39,38 +39,37 @@ UNRESERVED = (
     '0123456789'
     '_.-~'
 )
-"""Unreserved characters."""
+"""Unreserved characters specified in RFC 3986."""
 
 _URI_COMPONENTS = ('scheme', 'authority', 'path', 'query', 'fragment')
 
 _SCHEME_RE = re.compile(r"^[a-zA-Z][a-zA-Z0-9.+-]*$")
 
 _AUTHORITY_RE = re.compile(r"""
-\A
 (?:(.*)\@)?     # userinfo
-(.*?)           # host
+(.*?)           # host -- FIXME: lookahead?
 (?:\:(\d+))?    # port
-\Z
+$
 """, flags=(re.VERBOSE))
 
 
-def urisplit(uri):
+def urisplit(string):
     """Split a well-formed URI string into a tuple with five components
     corresponding to a URI's general structure::
 
       <scheme>://<authority>/<path>?<query>#<fragment>
 
-    The return value is an instance of a subclass of :class:`tuple`
-    with the following additional read-only attributes:
+    The return value is an instance of a subclass of
+    :class:`collections.namedtuple` with the following read-only
+    attributes:
 
     +-------------------+-------+---------------------------------------------+
     | Attribute         | Index | Value                                       |
     +===================+=======+=============================================+
     | :attr:`scheme`    | 0     | URI scheme, or :const:`None` if not present |
     +-------------------+-------+---------------------------------------------+
-    | :attr:`authority` | 1     | Authority component, or :const:`None` if    |
+    | :attr:`authority` | 1     | Authority component,                        |
     |                   |       | or :const:`None` if not present             |
-    |                   |       |                                             |
     +-------------------+-------+---------------------------------------------+
     | :attr:`path`      | 2     | Path component, always present but may be   |
     |                   |       | empty                                       |
@@ -87,12 +86,12 @@ def urisplit(uri):
     | :attr:`host`      |       | Host subcomponent of authority,             |
     |                   |       | or :const:`None` if not present             |
     +-------------------+-------+---------------------------------------------+
-    | :attr:`port`      |       | Port subcomponent of authority as integer,  |
+    | :attr:`port`      |       | Port subcomponent of authority,             |
     |                   |       | or :const:`None` if not present             |
     +-------------------+-------+---------------------------------------------+
 
     """
-    return SplitResult(*RE.match(uri).groups())
+    return SplitResult(*RE.match(string).groups())
 
 
 def uriunsplit(parts):
@@ -121,17 +120,29 @@ def urijoin(base, ref, strict=False):
     return uriunsplit(urisplit(base).transform(ref, strict))
 
 
-def uridefrag(uri):
-    """Remove an existing fragment from a URI string.
+def uridefrag(string):
+    """Remove an existing fragment component from a URI string.
 
-    Return a tuple of the defragmented URI and the fragment.  If `uri`
-    contains no fragment, the second element is :const:`None`.
+    The return value is an instance of a subclass of
+    :class:`collections.namedtuple` with the following read-only
+    attributes:
+
+    +-------------------+-------+---------------------------------------------+
+    | Attribute         | Index | Value                                       |
+    +===================+=======+=============================================+
+    | :attr:`base`      | 0     | The absoulte URI or relative URI reference  |
+    |                   |       | without the fragment identifier             |
+    +-------------------+-------+---------------------------------------------+
+    | :attr:`fragment`  | 1     | The fragment identifier,                    |
+    |                   |       | or :const:`None` if not present             |
+    +-------------------+-------+---------------------------------------------+
 
     """
-    if '#' in uri:
-        return DefragResult(*uri.split('#', 1))
+    parts = string.partition('#')
+    if parts[1]:
+        return DefragResult(parts[0], parts[2])
     else:
-        return DefragResult(uri, None)
+        return DefragResult(parts[0], None)
 
 
 def uriencode(string, safe='', encoding='utf-8'):
@@ -164,7 +175,7 @@ def urinormpath(path):
         elif out:
             out.pop()
     # Fix leading/trailing slashes
-    if path.startswith('/') and out[0] != '':
+    if path.startswith('/') and (not out or out[0]):
         out.insert(0, '')
     if path.endswith('/.') or path.endswith('/..'):
         out.append('')
@@ -186,11 +197,11 @@ def uricompose(scheme=None, authority=None, path='', query=None,
     # only produce lowercase scheme names for consistency.
     if scheme is not None:
         if not _SCHEME_RE.match(scheme):
-            raise ValueError('Invalid scheme %r' % scheme)
+            raise ValueError('Invalid scheme: %r' % scheme)
         scheme = scheme.lower()
 
     if authority is not None:
-        # TODO: check authority is well-formed, allow '[]' for IPv6
+        # TODO: check authority is well-formed, IPv6 support
         authority = uriencode(authority, SUB_DELIMS + ':@', encoding)
 
     # RFC 3986 3.3: If a URI contains an authority component, then the
@@ -220,11 +231,8 @@ def uricompose(scheme=None, authority=None, path='', query=None,
     return uriunsplit((scheme, authority, path, query, fragment))
 
 
-class SplitResult(namedtuple('SplitResult', _URI_COMPONENTS)):
-    """Extend :class:`collections.namedtuple` to hold :func:`urisplit`
-    results.
-
-    """
+class SplitResult(collections.namedtuple('SplitResult', _URI_COMPONENTS)):
+    """Class to hold :func:`urisplit` results."""
 
     @property
     def _splitauth(self):
@@ -243,13 +251,86 @@ class SplitResult(namedtuple('SplitResult', _URI_COMPONENTS)):
 
     @property
     def port(self):
-        port = self._splitauth[2]
-        return int(port, 10) if port is not None else None
+        return self._splitauth[2]
 
     def geturi(self):
         """Return the re-combined version of the original URI as a string."""
         return uriunsplit(self)
 
+    def getscheme(self, default=None):
+        """Return the URI scheme in canonical (lowercase) form, or `default`
+        if the original URI did not contain a scheme component.  Raise
+        a :class:`ValueError` if the scheme is not well-formed.
+
+        """
+        if self.scheme is None:
+            return default
+        if not _SCHEME_RE.match(self.scheme):
+            raise ValueError('Invalid scheme: %r' % self.scheme)
+        return self.scheme.lower()
+
+    def getauthority(self, default=None, encoding='utf-8'):
+        """Return the decoded URI authority, or `default` if the original URI
+        did not contain an authority component.
+
+        """
+        if self.authority is None:
+            return default
+        return uridecode(self.authority, encoding)
+
+    def getpath(self, encoding='utf-8'):
+        """Return the decoded URI path."""
+        return uridecode(self.path, encoding)
+
+    def getquery(self, default=None, encoding='utf-8'):
+        """Return the decoded query string, or `default` if the original URI
+        did not contain a query component.
+
+        """
+        if self.query is None:
+            return default
+        return uridecode(self.query, encoding)
+
+    def getfragment(self, default=None, encoding='utf-8'):
+        """Return the decoded fragment identifier, or `default` if the
+        original URI did not contain a fragment component.
+
+        """
+        if self.fragment is None:
+            return default
+        return uridecode(self.fragment, encoding)
+
+    def getuserinfo(self, default=None, encoding='utf-8'):
+        """Return the decoded userinfo subcomponent of the URI authority, or
+        `default` if the original URI did not contain a userinfo
+        field.
+
+        """
+        if self.userinfo is None:
+            return default
+        return uridecode(self.userinfo, encoding)
+
+    def gethost(self, default=None, encoding='utf-8'):
+        """Return the decoded host subcomponent of the URI authority, or
+        `default` if the original URI did not contain a host.
+
+        If the host represents an internationalized domain name
+        intended for resolution via DNS, the :const:`'idna'` encoding
+        must be specified to return a Unicode domain name.
+
+        """
+        if self.host is None:
+            return default
+        return uridecode(self.host, encoding)
+
+    def getport(self, default=None):
+        """Return the port subcomponent of the URI authority as an
+        :class:`int`, or `default` if the original URI did not contain
+        a port.
+
+        """
+        return int(self.port, 10) if self.port is not None else default
+
     def transform(self, ref, strict=False):
         """Convert a URI reference relative to `self` into a
         :class:`SplitResult` representing its target.
@@ -277,15 +358,28 @@ class SplitResult(namedtuple('SplitResult', _URI_COMPONENTS)):
         return SplitResult(self.scheme, self.authority, path, query, fragment)
 
 
-class DefragResult(namedtuple('DefragResult', 'uri fragment')):
-    """Extend :class:`collectionsnamedtuple` to hold :func:`uridefrag`
-    results.
-
-    """
+class DefragResult(collections.namedtuple('DefragResult', 'base fragment')):
+    """Class to hold :func:`uridefrag` results."""
 
     def geturi(self):
         """Return the re-combined version of the original URI as a string."""
         if self.fragment is not None:
-            return self.uri + '#' + self.fragment
+            return self.base + '#' + self.fragment
         else:
-            return self.uri
+            return self.base
+
+    def getbase(self, encoding='utf-8'):
+        """Return the decoded absolute URI or relative URI reference without
+        the fragment.
+
+        """
+        return uridecode(self.base, encoding)
+
+    def getfragment(self, default=None, encoding='utf-8'):
+        """Return the decoded fragment identifier, or `default` if the
+        original URI did not contain a fragment component.
+
+        """
+        if self.fragment is None:
+            return default
+        return uridecode(self.fragment, encoding)
